@@ -27,6 +27,8 @@ import os
 import sys
 import traceback
 
+from libqtile.log_utils import logger
+
 
 class ConfigError(Exception):
     pass
@@ -52,15 +54,20 @@ class File(object):
                 sys.path.insert(0, os.path.dirname(fname))
                 config = __import__(os.path.basename(fname)[:-3])
             except Exception as v:
+
+                tb = traceback.format_exc()
+
                 # On restart, user potentially has some windows open, but they
                 # screwed up their config. So as not to lose their apps, we
                 # just load the default config here.
                 if is_restart:
-                    traceback.print_exc()
+                    logger.warning(
+                        'Caught exception in configuration:\n\n'
+                        '%s\n\n'
+                        'Qtile restarted with default config', tb)
                     config = None
                 else:
-                    tb = traceback.format_exc()
-                    raise ConfigError(str(v) + "\n\n" + tb)
+                    raise ConfigError(tb)
         else:
             config = None
 
@@ -73,6 +80,7 @@ class File(object):
             "dgroups_key_binder",
             "dgroups_app_rules",
             "follow_mouse_focus",
+            "focus_on_window_activation",
             "cursor_warp",
             "layouts",
             "floating_layout",
