@@ -33,13 +33,13 @@ from .base import Layout
 
 class Matrix(Layout):
     """
-        This layout divides the screen into a matrix of equally sized cells
-        and places one window in each cell. The number of columns is
-        configurable and can also be changed interactively.
+    This layout divides the screen into a matrix of equally sized cells and
+    places one window in each cell. The number of columns is configurable and
+    can also be changed interactively.
     """
     defaults = [
         ("border_focus", "#0000ff", "Border colour for the focused window."),
-        ("border_normal", "#000000", "Border colour for un-focused winows."),
+        ("border_normal", "#000000", "Border colour for un-focused windows."),
         ("border_width", 1, "Border width."),
         ("name", "matrix", "Name of this layout."),
         ("margin", 0, "Margin of the layout"),
@@ -112,8 +112,7 @@ class Matrix(Layout):
     def focus_next(self, window):
         if not self.clients:
             return
-        if self.get_current_window != window:
-            self.focus(window)
+
         idx = self.clients.index(window)
         if idx + 1 < len(self.clients):
             return self.clients[idx + 1]
@@ -121,8 +120,7 @@ class Matrix(Layout):
     def focus_previous(self, window):
         if not self.clients:
             return
-        if self.get_current_window != window:
-            self.focus(window)
+
         idx = self.clients.index(window)
         if idx > 0:
             return self.clients[idx - 1]
@@ -159,61 +157,49 @@ class Matrix(Layout):
     def cmd_next(self):
         client = self.focus_next(self.get_current_window()) or \
             self.focus_first()
-        self.group.focus(client, False)
+        self.group.focus(client)
 
     def cmd_previous(self):
         client = self.focus_previous(self.get_current_window()) or \
             self.focus_last()
-        self.group.focus(client, False)
+        self.group.focus(client)
 
     def cmd_left(self):
-        """
-            Switch to the next window on current row
-        """
+        """Switch to the next window on current row"""
         column, row = self.current_window
         self.current_window = ((column - 1) % len(self.get_row(row)), row)
-        self.group.focus(self.get_current_window(), False)
+        self.group.focus(self.get_current_window())
 
     def cmd_right(self):
-        """
-            Switch to the next window on current row
-        """
+        """Switch to the next window on current row"""
         column, row = self.current_window
         self.current_window = ((column + 1) % len(self.get_row(row)), row)
-        self.group.focus(self.get_current_window(), False)
+        self.group.focus(self.get_current_window())
 
     def cmd_down(self):
-        """
-            Switch to the next window in current column
-        """
+        """Switch to the next window in current column"""
         column, row = self.current_window
         self.current_window = (
             column,
             (row + 1) % len(self.get_column(column))
         )
-        self.group.focus(self.get_current_window(), False)
+        self.group.focus(self.get_current_window())
 
     def cmd_up(self):
-        """
-            Switch to the previous window in current column
-        """
+        """Switch to the previous window in current column"""
         column, row = self.current_window
         self.current_window = (
             column,
             (row - 1) % len(self.get_column(column))
         )
-        self.group.focus(self.get_current_window(), False)
+        self.group.focus(self.get_current_window())
 
     def cmd_delete(self):
-        """
-            Decrease number of columns
-        """
+        """Decrease number of columns"""
         self.columns -= 1
         self.group.layoutAll()
 
     def cmd_add(self):
-        """
-            Increase number of columns
-        """
+        """Increase number of columns"""
         self.columns += 1
         self.group.layoutAll()
