@@ -37,7 +37,7 @@ from .. import utils
 class Tile(Layout):
     defaults = [
         ("border_focus", "#0000ff", "Border colour for the focused window."),
-        ("border_normal", "#000000", "Border colour for un-focused winows."),
+        ("border_normal", "#000000", "Border colour for un-focused windows."),
         ("border_width", 1, "Border width."),
         ("name", "tile", "Name of this layout."),
         ("margin", 0, "Margin of the layout"),
@@ -218,11 +218,11 @@ class Tile(Layout):
 
     def cmd_next(self):
         client = self.focus_next(self.focused) or self.focus_first()
-        self.group.focus(client, False)
+        self.group.focus(client)
 
     def cmd_previous(self):
         client = self.focus_previous(self.focused) or self.focus_last()
-        self.group.focus(client, False)
+        self.group.focus(client)
 
     def cmd_decrease_ratio(self):
         self.ratio -= self.ratio_increment
