@@ -1,4 +1,4 @@
-# -*- coding:utf-8 -*-
+# -*- coding: utf-8 -*-
 # Copyright (c) 2011-2012 Dustin Lacewell
 # Copyright (c) 2011 Mounier Florian
 # Copyright (c) 2012 Craig Barnes
@@ -32,21 +32,21 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from .base import SingleWindow
+from __future__ import division
+
+from .base import Layout
 import math
 
 
-class MonadTall(SingleWindow):
-    """
-    This layout attempts to emulate the behavior of XMonad's default
-    tiling scheme.
+class MonadTall(Layout):
+    """Emulate the behavior of XMonad's default tiling scheme
 
     Main-Pane:
 
-    A main pane that contains a single window takes up a vertical
-    portion of the screen based on the ratio setting. This ratio can
-    be adjusted with the ``cmd_grow`` and ``cmd_shrink`` methods while
-    the main pane is in focus.
+    A main pane that contains a single window takes up a vertical portion of
+    the screen based on the ratio setting. This ratio can be adjusted with the
+    ``cmd_grow_main`` and ``cmd_shrink_main`` or, while the main pane is in
+    focus, ``cmd_grow`` and ``cmd_shrink``.
 
     ::
 
@@ -59,9 +59,8 @@ class MonadTall(SingleWindow):
         |            |      |
         ---------------------
 
-    Using the ``cmd_flip`` method will switch which horizontal side the
-    main pane will occupy. The main pane is considered the "top" of
-    the stack.
+    Using the ``cmd_flip`` method will switch which horizontal side the main
+    pane will occupy. The main pane is considered the "top" of the stack.
 
     ::
 
@@ -76,11 +75,11 @@ class MonadTall(SingleWindow):
 
     Secondary-panes:
 
-    Occupying the rest of the screen are one or more secondary panes.
-    The secondary panes will share the vertical space of the screen
-    however they can be resized at will with the ``cmd_grow`` and
-    ``cmd_shrink`` methods. The other secondary panes will adjust their
-    sizes to smoothly fill all of the space.
+    Occupying the rest of the screen are one or more secondary panes.  The
+    secondary panes will share the vertical space of the screen however they
+    can be resized at will with the ``cmd_grow`` and ``cmd_shrink`` methods.
+    The other secondary panes will adjust their sizes to smoothly fill all of
+    the space.
 
     ::
 
@@ -94,8 +93,8 @@ class MonadTall(SingleWindow):
         ---------------------          ---------------------
 
     Panes can be moved with the ``cmd_shuffle_up`` and ``cmd_shuffle_down``
-    methods. As mentioned the main pane is considered the top of the
-    stack; moving up is counter-clockwise and moving down is clockwise.
+    methods. As mentioned the main pane is considered the top of the stack;
+    moving up is counter-clockwise and moving down is clockwise.
 
     The opposite is true if the layout is "flipped".
 
@@ -113,18 +112,15 @@ class MonadTall(SingleWindow):
 
     Normalizing:
 
-    To restore all client windows to their default size ratios simply
-    use the ``cmd_normalize`` method.
-
+    To restore all client windows to their default size ratios simply use the
+    ``cmd_normalize`` method.
 
     Maximizing:
 
     To toggle a client window between its minimum and maximum sizes
     simply use the ``cmd_maximize`` on a focused client.
 
-    Suggested Bindings:
-
-    ::
+    Suggested Bindings::
 
         Key([modkey], "h", lazy.layout.left()),
         Key([modkey], "l", lazy.layout.right()),
@@ -151,8 +147,9 @@ class MonadTall(SingleWindow):
 
     defaults = [
         ("border_focus", "#ff0000", "Border colour for the focused window."),
-        ("border_normal", "#000000", "Border colour for un-focused winows."),
+        ("border_normal", "#000000", "Border colour for un-focused windows."),
         ("border_width", 2, "Border width."),
+        ("single_border_width", None, "Border width for single window"),
         ("name", "xmonad-tall", "Name of this layout."),
         ("margin", 0, "Margin of the layout"),
         ("ratio", _med_ratio,
@@ -165,8 +162,10 @@ class MonadTall(SingleWindow):
     ]
 
     def __init__(self, **config):
-        SingleWindow.__init__(self, **config)
+        Layout.__init__(self, **config)
         self.add_defaults(MonadTall.defaults)
+        if self.single_border_width is None:
+            self.single_border_width = self.border_width
         self.clients = []
         self.relative_sizes = []
         self._focus = 0
@@ -199,7 +198,7 @@ class MonadTall(SingleWindow):
 
     def clone(self, group):
         "Clone layout for other groups"
-        c = SingleWindow.clone(self, group)
+        c = Layout.clone(self, group)
         c.clients = []
         c.sizes = []
         c.relative_sizes = []
@@ -303,9 +302,9 @@ class MonadTall(SingleWindow):
             client.place(
                 self.group.screen.dx,
                 self.group.screen.dy,
-                self.group.screen.dwidth,
-                self.group.screen.dheight,
-                0,
+                self.group.screen.dwidth - 2 * self.single_border_width,
+                self.group.screen.dheight - 2 * self.single_border_width,
+                self.single_border_width,
                 px,
                 margin=self.margin,
             )
@@ -353,6 +352,10 @@ class MonadTall(SingleWindow):
             height = self._get_absolute_size_from_relative(
                 self.relative_sizes[cidx - 1]
             )
+            # fix double margin
+            if cidx > 1:
+                ypos -= self.margin
+                height += self.margin
             # place client based on calculated dimensions
             client.place(
                 xpos,
@@ -368,15 +371,22 @@ class MonadTall(SingleWindow):
             # main client
             width = width_main - 2 * self.border_width
             client.place(
-                xpos, self.group.screen.dy,
-                width,
-                self.group.screen.dheight - 2 * self.border_width,
+                xpos + self.margin,
+                self.group.screen.dy + self.margin,
+                width - self.margin,
+                self.group.screen.dheight - 2 * self.border_width - 2 * self.margin,
                 self.border_width,
                 px,
-                margin=self.margin,
             )
             client.unhide()
 
+    def info(self):
+        return {
+            'clients': [c.name for c in self.clients],
+            'main': self.clients[0].name if self.clients else None,
+            'secondary': [c.name for c in self.clients[1:]]
+        }
+
     def get_shrink_margin(self, cidx):
         "Return how many remaining pixels a client can shrink"
         return max(
@@ -387,10 +397,10 @@ class MonadTall(SingleWindow):
         )
 
     def shrink(self, cidx, amt):
-        """
-        Reduce the size of a client. Will only shrink the client
-        until it reaches the configured minimum size. Any amount
-        that was prevented in the resize is returned.
+        """Reduce the size of a client
+
+        Will only shrink the client until it reaches the configured minimum
+        size. Any amount that was prevented in the resize is returned.
         """
         # get max resizable amount
         margin = self.get_shrink_margin(cidx)
@@ -404,13 +414,13 @@ class MonadTall(SingleWindow):
             return 0
 
     def shrink_up(self, cidx, amt):
-        """
-        Will shrink all secondary clients above the specified
-        index in order. Each client will attempt to shrink as
-        much as it is able before the next client is resized.
+        """Shrink the window up
 
-        Any amount that was unable to be applied to the
-        clients is returned.
+        Will shrink all secondary clients above the specified index in order.
+        Each client will attempt to shrink as much as it is able before the
+        next client is resized.
+
+        Any amount that was unable to be applied to the clients is returned.
         """
         left = amt  # track unused shrink amount
         # for each client before specified index
@@ -421,15 +431,14 @@ class MonadTall(SingleWindow):
         return left
 
     def shrink_up_shared(self, cidx, amt):
-        """
-        Will shrink all secondary clients above the specified
-        index by an equal share of the provided amount. After
-        applying the shared amount to all affected clients,
-        any amount left over will be applied in a
-        non-equal manner with ``shrink_up``.
+        """Shrink the shared space
 
-        Any amount that was unable to be applied to the
-        clients is returned.
+        Will shrink all secondary clients above the specified index by an equal
+        share of the provided amount. After applying the shared amount to all
+        affected clients, any amount left over will be applied in a non-equal
+        manner with ``shrink_up``.
+
+        Any amount that was unable to be applied to the clients is returned.
         """
         # split shrink amount among number of clients
         per_amt = amt / cidx
@@ -445,13 +454,13 @@ class MonadTall(SingleWindow):
         return left
 
     def shrink_down(self, cidx, amt):
-        """
-        Will shrink all secondary clients below the specified
-        index in order. Each client will attempt to shrink as
-        much as it is able before the next client is resized.
+        """Shrink current window down
 
-        Any amount that was unable to be applied to the
-        clients is returned.
+        Will shrink all secondary clients below the specified index in order.
+        Each client will attempt to shrink as much as it is able before the
+        next client is resized.
+
+        Any amount that was unable to be applied to the clients is returned.
         """
         left = amt  # track unused shrink amount
         # for each client after specified index
@@ -462,15 +471,14 @@ class MonadTall(SingleWindow):
         return left
 
     def shrink_down_shared(self, cidx, amt):
-        """
-        Will shrink all secondary clients below the specified
-        index by an equal share of the provided amount. After
-        applying the shared amount to all affected clients,
-        any amount left over will be applied in a
-        non-equal manner with ``shrink_down``.
+        """Shrink secondary clients
 
-        Any amount that was unable to be applied to the
-        clients is returned.
+        Will shrink all secondary clients below the specified index by an equal
+        share of the provided amount. After applying the shared amount to all
+        affected clients, any amount left over will be applied in a non-equal
+        manner with ``shrink_down``.
+
+        Any amount that was unable to be applied to the clients is returned.
         """
         # split shrink amount among number of clients
         per_amt = amt / (len(self.relative_sizes) - 1 - cidx)
@@ -479,33 +487,24 @@ class MonadTall(SingleWindow):
         for idx in range(cidx + 1, len(self.relative_sizes)):
             # shrink by equal amount and track left-over
             left -= per_amt - self.shrink(idx, per_amt)
-        # apply non-equal shinkage secondary pass
+        # apply non-equal shrinkage secondary pass
         # in order to use up any left over shrink amounts
         left = self.shrink_down(cidx, left)
         # return whatever could not be applied
         return left
 
     def _grow_main(self, amt):
-        """
-        Will grow the client that is currently
-        in the main pane.
-        """
+        """Will grow the client that is currently in the main pane"""
         self.ratio += amt
         self.ratio = min(self._max_ratio, self.ratio)
 
     def _grow_solo_secondary(self, amt):
-        """
-        Will grow the solitary client in the
-        secondary pane.
-        """
+        """Will grow the solitary client in the secondary pane"""
         self.ratio -= amt
         self.ratio = max(self._min_ratio, self.ratio)
 
     def _grow_secondary(self, amt):
-        """
-        Will grow the focused client in the
-        secondary pane.
-        """
+        """Will grow the focused client in the secondary pane"""
         half_change_size = amt / 2
         # track unshrinkable amounts
         left = amt
@@ -545,14 +544,12 @@ class MonadTall(SingleWindow):
             self._get_relative_size_from_absolute(diff)
 
     def cmd_grow(self):
+        """Grow current window
+
+        Will grow the currently focused client reducing the size of those
+        around it. Growing will stop when no other secondary clients can reduce
+        their size any further.
         """
-        Will grow the currently focused client reducing the
-        size of those around it. Growing will stop when no
-        other secondary clients can reduce their size any
-        further.
-        """
-        # get currently focused client
-        self.clients[self.focused]
         if self.focused == 0:
             self._grow_main(self.change_ratio)
         elif len(self.clients) == 2:
@@ -561,14 +558,33 @@ class MonadTall(SingleWindow):
             self._grow_secondary(self.change_size)
         self.group.layoutAll()
 
+    def cmd_grow_main(self):
+        """Grow main pane
+
+        Will grow the main pane, reducing the size of clients in the secondary
+        pane.
+        """
+        self._grow_main(self.change_ratio)
+        self.group.layoutAll()
+
+    def cmd_shrink_main(self):
+        """Shrink main pane
+
+        Will shrink the main pane, increasing the size of clients in the
+        secondary pane.
+        """
+        self._shrink_main(self.change_ratio)
+        self.group.layoutAll()
+
     def grow(self, cidx, amt):
         "Grow secondary client by specified amount"
         self.relative_sizes[cidx] += self._get_relative_size_from_absolute(amt)
 
     def grow_up_shared(self, cidx, amt):
-        """
-        Will grow all secondary clients above the specified
-        index by an equal share of the provided amount.
+        """Grow higher secondary clients
+
+        Will grow all secondary clients above the specified index by an equal
+        share of the provided amount.
         """
         # split grow amount among number of clients
         per_amt = amt / cidx
@@ -576,9 +592,10 @@ class MonadTall(SingleWindow):
             self.grow(idx, per_amt)
 
     def grow_down_shared(self, cidx, amt):
-        """
-        Will grow all secondary clients below the specified
-        index by an equal share of the provided amount.
+        """Grow lower secondary clients
+
+        Will grow all secondary clients below the specified index by an equal
+        share of the provided amount.
         """
         # split grow amount among number of clients
         per_amt = amt / (len(self.relative_sizes) - 1 - cidx)
@@ -586,26 +603,17 @@ class MonadTall(SingleWindow):
             self.grow(idx, per_amt)
 
     def _shrink_main(self, amt):
-        """
-        Will shrink the client that currently
-        in the main pane.
-        """
+        """Will shrink the client that currently in the main pane"""
         self.ratio -= amt
         self.ratio = max(self._min_ratio, self.ratio)
 
     def _shrink_solo_secondary(self, amt):
-        """
-        Will shrink the solitary client in the
-        secondary pane.
-        """
+        """Will shrink the solitary client in the secondary pane"""
         self.ratio += amt
         self.ratio = min(self._max_ratio, self.ratio)
 
     def _shrink_secondary(self, amt):
-        """
-        Will shrink the focused client in the
-        secondary pane.
-        """
+        """Will shrink the focused client in the secondary pane"""
         # get focused client
         client = self.clients[self.focused]
 
@@ -651,36 +659,34 @@ class MonadTall(SingleWindow):
     def focus_next(self, window):
         if not self.clients:
             return
-        if self.focused != self.clients.index(window):
-            self.focus(window)
+
         if self.focused + 1 < len(self.clients):
             return self.clients[self.focused + 1]
 
     def focus_previous(self, window):
         if not self.clients:
             return
-        if self.focused != self.clients.index(window):
-            self.focus(window)
+
         if self.focused > 0:
             return self.clients[self.focused - 1]
 
     def cmd_next(self):
         client = self.focus_next(self.clients[self.focused]) or \
             self.focus_first()
-        self.group.focus(client, False)
+        self.group.focus(client)
 
     def cmd_previous(self):
         client = self.focus_previous(self.clients[self.focused]) or \
             self.focus_last()
-        self.group.focus(client, False)
+        self.group.focus(client)
 
     def cmd_shrink(self):
+        """Shrink current window
+
+        Will shrink the currently focused client reducing the size of those
+        around it. Shrinking will stop when the client has reached the minimum
+        size.
         """
-        Will shrink the currently focused client reducing the
-        size of those around it. Shrinking will stop when the
-        client has reached the minimum size.
-        """
-        self.clients[self.focused]
         if self.focused == 0:
             self._shrink_main(self.change_ratio)
         elif len(self.clients) == 2:
@@ -690,40 +696,40 @@ class MonadTall(SingleWindow):
         self.group.layoutAll()
 
     def cmd_up(self):
-        "Focus on the next more prominent client on the stack"
+        """Focus on the next more prominent client on the stack"""
         self.focused -= 1
-        self.group.focus(self.clients[self.focused], False)
+        self.group.focus(self.clients[self.focused])
 
     def cmd_down(self):
-        "Focus on the less prominent client on the stack"
+        """Focus on the less prominent client on the stack"""
         self.focused += 1
-        self.group.focus(self.clients[self.focused], False)
+        self.group.focus(self.clients[self.focused])
 
     def cmd_shuffle_up(self):
-        "Shuffle the client up the stack."
+        """Shuffle the client up the stack"""
         _oldf = self.focused
         self.focused -= 1
         self.clients[_oldf], self.clients[self.focused] = \
             self.clients[self.focused], self.clients[_oldf]
         self.group.layoutAll()
-        self.group.focus(self.clients[self.focused], False)
+        self.group.focus(self.clients[self.focused])
 
     def cmd_shuffle_down(self):
-        "Shuffle the client down the stack."
+        """Shuffle the client down the stack"""
         _oldf = self.focused
         self.focused += 1
         self.clients[_oldf], self.clients[self.focused] = \
             self.clients[self.focused], self.clients[_oldf]
         self.group.layoutAll()
-        self.group.focus(self.clients[self.focused], False)
+        self.group.focus(self.clients[self.focused])
 
     def cmd_flip(self):
-        "Flip the layout horizontally."
+        """Flip the layout horizontally"""
         self.align = self._left if self.align == self._right else self._right
         self.group.layoutAll()
 
     def _get_closest(self, x, y, clients):
-        "Get closest window to a point x,y"
+        """Get closest window to a point x,y"""
         target = min(
             clients,
             key=lambda c: math.hypot(c.info()['x'] - x, c.info()['y'] - y)
@@ -731,17 +737,17 @@ class MonadTall(SingleWindow):
         return target
 
     def cmd_swap(self, window1, window2):
-        "Swap two windows."
+        """Swap two windows"""
         index1 = self.clients.index(window1)
         index2 = self.clients.index(window2)
         self.clients[index1], self.clients[index2] = \
             self.clients[index2], self.clients[index1]
         self.group.layoutAll()
         self.focused = index1
-        self.group.focus(window1, False)
+        self.group.focus(window1)
 
     def cmd_swap_left(self):
-        "Swap current window with closest window to the left."
+        """Swap current window with closest window to the left"""
         x = self._get_window().x
         y = self._get_window().y
         candidates = [c for c in self.clients if c.info()['x'] < x]
@@ -749,7 +755,7 @@ class MonadTall(SingleWindow):
         self.cmd_swap(self._get_window(), target)
 
     def cmd_swap_right(self):
-        "Swap current window with closest window to the right."
+        """Swap current window with closest window to the right"""
         x = self._get_window().x
         y = self. _get_window().y
         candidates = [c for c in self.clients if c.info()['x'] > x]
@@ -757,26 +763,26 @@ class MonadTall(SingleWindow):
         self.cmd_swap(self._get_window(), target)
 
     def cmd_swap_main(self):
-        "Swap current window to main pane."
+        """Swap current window to main pane"""
         if self.align == self._left:
             self.cmd_swap_left()
         elif self.align == self._right:
             self.cmd_swap_right()
 
     def cmd_left(self):
-        "Focus on the closest window to the left of the current window."
+        """Focus on the closest window to the left of the current window"""
         x = self._get_window().x
         y = self._get_window().y
         candidates = [c for c in self.clients if c.info()['x'] < x]
         target = self._get_closest(x, y, candidates)
         self.focused = self.clients.index(target)
-        self.group.focus(self.clients[self.focused], False)
+        self.group.focus(self.clients[self.focused])
 
     def cmd_right(self):
-        "Focus on the closest window to the right of the current window."
+        """Focus on the closest window to the right of the current window"""
         x = self._get_window().x
         y = self._get_window().y
         candidates = [c for c in self.clients if c.info()['x'] > x]
         target = self._get_closest(x, y, candidates)
         self.focused = self.clients.index(target)
-        self.group.focus(self.clients[self.focused], False)
+        self.group.focus(self.clients[self.focused])
