@@ -26,7 +26,7 @@
     If dbus is available, this module implements a
     org.freedesktop.Notifications service.
 """
-import logging
+from .log_utils import logger
 
 try:
     import dbus
@@ -91,13 +91,13 @@ class NotificationManager(object):
                 DBusGMainLoop(set_as_default=True)
                 self._service = NotificationService(self)
             except Exception:
-                logging.getLogger('qtile').exception('Dbus connection failed')
+                logger.exception('Dbus connection failed')
                 self._service = None
         return self._service
 
     def register(self, callback):
         if not self.service:
-            logging.getLogger('qtile').warning(
+            logger.warning(
                 'Registering %s without any dbus connection existing',
                 callback.__name__,
             )
