@@ -20,12 +20,10 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import logging
 import traceback
 import importlib
 
-
-logger = logging.getLogger('qtile')
+from libqtile.log_utils import logger
 
 
 def safe_import(module_name, class_name):
@@ -38,6 +36,7 @@ def safe_import(module_name, class_name):
             safe_import(module_name, name)
         return
     package = __package__
+    # TODO: remove when we really want to drop 3.2 support
     # python 3.2 don't set __package__
     if not package:
         package = __name__
@@ -45,15 +44,15 @@ def safe_import(module_name, class_name):
         module = importlib.import_module(module_name, package)
         globals()[class_name] = getattr(module, class_name)
     except ImportError as error:
-        msg = "Can't Import Widget: '%s.%s', %s"
-        logger.warning(msg % (module_name, class_name, error))
-        logger.debug(traceback.format_exc())
+        logger.warning("Can't Import Widget: '%s.%s', %s", module_name, class_name, error)
+        logger.debug("%s", traceback.format_exc())
 
 
 safe_import(".backlight", "Backlight")
 safe_import(".battery", ["Battery", "BatteryIcon"])
 safe_import(".clock", "Clock")
 safe_import(".currentlayout", "CurrentLayout")
+safe_import(".currentscreen", "CurrentScreen")
 safe_import(".debuginfo", "DebugInfo")
 safe_import(".graph", ["CPUGraph", "MemoryGraph", "SwapGraph", "NetGraph",
                        "HDDGraph", "HDDBusyGraph"])
@@ -87,10 +86,12 @@ safe_import(".mpdwidget", "Mpd")
 safe_import(".yahoo_weather", "YahooWeather")
 safe_import(".bitcoin_ticker", "BitcoinTicker")
 safe_import(".wlan", "Wlan")
-safe_import(".google_calendar", "GoogleCalendar")
+safe_import(".khal_calendar", "KhalCalendar")
 safe_import(".imapwidget", "ImapWidget")
 safe_import(".net", "Net")
 safe_import(".keyboardkbdd", "KeyboardKbdd")
 safe_import(".cmus", "Cmus")
 safe_import(".wallpaper", "Wallpaper")
 safe_import(".check_updates", "CheckUpdates")
+safe_import(".moc", "Moc")
+safe_import(".memory", "Memory")
