@@ -24,15 +24,15 @@
 import os
 from . import base
 
+from libqtile.log_utils import logger
+
 BACKLIGHT_DIR = '/sys/class/backlight'
 
 FORMAT = '{percent: 2.0%}'
 
 
 class Backlight(base.InLoopPollText):
-    """
-        A simple widget to show the current brightness of a monitor.
-    """
+    """A simple widget to show the current brightness of a monitor"""
 
     filenames = {}
 
@@ -53,21 +53,22 @@ class Backlight(base.InLoopPollText):
             'maximum brightness in /sys/class/backlight/backlight_name'
         ),
         ('update_interval', .2, 'The delay in seconds between updates'),
+        ('step', 10, 'Percent of backlight every scroll changed')
     ]
 
     def __init__(self, **config):
         base.InLoopPollText.__init__(self, **config)
         self.add_defaults(Backlight.defaults)
+        self.future = None
 
     def _load_file(self, name):
         try:
             path = os.path.join(BACKLIGHT_DIR, self.backlight_name, name)
             with open(path, 'r') as f:
                 return f.read().strip()
-        except IOError:
-            return False
-        except Exception:
-            self.log.exception("Failed to get %s" % name)
+        except:
+            logger.exception("Failed to read file: %s" % name)
+            raise
 
     def _get_info(self):
         try:
@@ -75,14 +76,33 @@ class Backlight(base.InLoopPollText):
                 'brightness': float(self._load_file(self.brightness_file)),
                 'max': float(self._load_file(self.max_brightness_file)),
             }
-        except TypeError:
-            return False
+        except:
+            return
         return info
 
     def poll(self):
         info = self._get_info()
-        if info is False:
+        if not info:
             return 'Error'
 
         percent = info['brightness'] / info['max']
         return FORMAT.format(percent=percent)
+
+    def change_backlight(self, value):
+        self.call_process(["xbacklight", "-set", str(value)])
+
+    def button_press(self, x, y, button):
+        if self.future and not self.future.done():
+            return
+        info = self._get_info()
+        if info is False:
+            new = now = 100
+        else:
+            new = now = info["brightness"] / info["max"] * 100
+        if button == 5:  # down
+            new = max(now - self.step, 0)
+        elif button == 4:  # up
+            new = min(now + self.step, 100)
+        if new != now:
+            self.future = self.qtile.run_in_executor(self.change_backlight,
+                                                     new)
