@@ -29,10 +29,10 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from libqtile.log_utils import logger
 from .. import command, bar, configurable, drawer, confreader
 import six
 import subprocess
-import logging
 import threading
 import warnings
 
@@ -76,17 +76,18 @@ ORIENTATION_BOTH = _Orientations(3, 'horizontal and vertical')
 
 
 class _Widget(command.CommandObject, configurable.Configurable):
-    """
-        If length is set to the special value bar.STRETCH, the bar itself
-        will set the length to the maximum remaining space, after all other
-        widgets have been configured. Only ONE widget per bar can have the
-        bar.STRETCH length set.
+    """Base Widget class
 
-        In horizontal bars, 'length' corresponds to the width of the widget;
-        in vertical bars, it corresponds to the widget's height.
+    If length is set to the special value `bar.STRETCH`, the bar itself will
+    set the length to the maximum remaining space, after all other widgets have
+    been configured. Only ONE widget per bar can have the `bar.STRETCH` length
+    set.
 
-        The offsetx and offsety attributes are set by the Bar after all widgets
-        have been configured.
+    In horizontal bars, 'length' corresponds to the width of the widget; in
+    vertical bars, it corresponds to the widget's height.
+
+    The offsetx and offsety attributes are set by the Bar after all widgets
+    have been configured.
     """
     orientations = ORIENTATION_BOTH
     offsetx = None
@@ -102,8 +103,6 @@ class _Widget(command.CommandObject, configurable.Configurable):
         if "name" in config:
             self.name = config["name"]
 
-        self.log = logging.getLogger('qtile')
-
         configurable.Configurable.__init__(self, **config)
         self.add_defaults(_Widget.defaults)
 
@@ -180,6 +179,11 @@ class _Widget(command.CommandObject, configurable.Configurable):
             self.configured = True
             self.qtile.call_soon(self.timer_setup)
 
+    def finalize(self):
+        if hasattr(self, 'layout') and self.layout:
+            self.layout.finalize()
+        self.drawer.finalize()
+
     def clear(self):
         self.drawer.set_source_rgb(self.bar.background)
         self.drawer.fillrect(self.offsetx, self.offsety, self.width,
@@ -263,7 +267,7 @@ class _Widget(command.CommandObject, configurable.Configurable):
         try:
             method(*method_args)
         except:
-            self.log.exception('got exception from widget timer')
+            logger.exception('got exception from widget timer')
 
 
 UNSPECIFIED = bar.Obj("UNSPECIFIED")
@@ -461,8 +465,11 @@ class ThreadedPollText(InLoopPollText):
 
     def tick(self):
         def worker():
-            text = self.poll()
-            self.qtile.call_soon_threadsafe(self.update, text)
+            try:
+                text = self.poll()
+                self.qtile.call_soon_threadsafe(self.update, text)
+            except:
+                logger.exception("problem polling to update widget %s", self.name)
         # TODO: There are nice asyncio constructs for this sort of thing, I
         # think...
         threading.Thread(target=worker).start()
@@ -480,26 +487,37 @@ class ThreadPoolText(_TextBox):
 
     param: text - Initial text to display.
     """
+    defaults = [
+        ("update_interval", None, "Update interval in seconds, if none, the "
+            "widget updates whenever it's done'."),
+    ]
+
     def __init__(self, text, **config):
         super(ThreadPoolText, self).__init__(text, width=bar.CALCULATED,
                                              **config)
+        self.add_defaults(ThreadPoolText.defaults)
 
     def timer_setup(self):
         def on_done(future):
             try:
                 result = future.result()
             except Exception:
-                self.log.exception('poll() raised exceptions, not '
-                                   'rescheduling')
+                result = None
+                logger.exception('poll() raised exceptions, not rescheduling')
 
             if result is not None:
                 try:
                     self.update(result)
-                    self.timer_setup()
+
+                    if self.update_interval is not None:
+                        self.timeout_add(self.update_interval, self.timer_setup)
+                    else:
+                        self.timer_setup()
+
                 except Exception:
-                    self.log.exception('Failed to reschedule.')
+                    logger.exception('Failed to reschedule.')
             else:
-                self.log.warning('poll() returned None, not rescheduling')
+                logger.warning('poll() returned None, not rescheduling')
 
         future = self.qtile.run_in_executor(self.poll)
         future.add_done_callback(on_done)
@@ -523,12 +541,11 @@ class ThreadPoolText(_TextBox):
 
 
 class PaddingMixin(object):
-    """
-        Mixin that provides padding(_x|_y|)
+    """Mixin that provides padding(_x|_y|)
 
-        To use it, subclass and add this to __init__:
+    To use it, subclass and add this to __init__:
 
-            self.add_defaults(base.PaddingMixin.defaults)
+        self.add_defaults(base.PaddingMixin.defaults)
     """
 
     defaults = [
@@ -542,12 +559,11 @@ class PaddingMixin(object):
 
 
 class MarginMixin(object):
-    """
-        Mixin that provides margin(_x|_y|)
+    """Mixin that provides margin(_x|_y|)
 
-        To use it, subclass and add this to __init__:
+    To use it, subclass and add this to __init__:
 
-            self.add_defaults(base.MarginMixin.defaults)
+        self.add_defaults(base.MarginMixin.defaults)
     """
 
     defaults = [
