@@ -34,6 +34,7 @@ from __future__ import division
 import cairocffi
 import os
 from libqtile import bar
+from libqtile.log_utils import logger
 from . import base
 
 BAT_DIR = '/sys/class/power_supply'
@@ -57,7 +58,7 @@ def default_icon_path():
 
 
 class _Battery(base._TextBox):
-    ''' Base battery class '''
+    """Base battery class"""
 
     filenames = {}
 
@@ -104,7 +105,7 @@ class _Battery(base._TextBox):
                 return 0
             return False
         except Exception:
-            self.log.exception("Failed to get %s" % name)
+            logger.exception("Failed to get %s" % name)
 
     def _get_param(self, name):
         if name in self.filenames and self.filenames[name]:
@@ -122,7 +123,7 @@ class _Battery(base._TextBox):
             # Iterate over the possibilities, and return the first valid value
             for file in file_list:
                 value = self._load_file(file)
-                if not (value in (False, None)):
+                if value is not False and value is not None:
                     self.filenames[name] = file
                     return value
 
@@ -146,30 +147,25 @@ class _Battery(base._TextBox):
 
 
 class Battery(_Battery):
-    """
-        A simple but flexible text-based battery widget.
-    """
+    """A simple but flexible text-based battery widget"""
     orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
-        ('low_foreground', 'FF0000', 'font color when battery is low'),
-        (
-            'format',
-            '{char} {percent:2.0%} {hour:d}:{min:02d}',
-            'Display format'
-        ),
         ('charge_char', '^', 'Character to indicate the battery is charging'),
-        (
-            'discharge_char',
-            'V',
-            'Character to indicate the battery'
-            ' is discharging'
-        ),
-        (
-            'low_percentage',
-            0.10,
-            "0 < x < 1 at which to indicate battery is low with low_foreground"
-        ),
+        ('discharge_char',
+         'V',
+         'Character to indicate the battery is discharging'
+         ),
+        ('error_message', 'Error', 'Error message if something is wrong'),
+        ('format',
+         '{char} {percent:2.0%} {hour:d}:{min:02d}',
+         'Display format'
+         ),
         ('hide_threshold', None, 'Hide the text when there is enough energy'),
+        ('low_percentage',
+         0.10,
+         "Indicates when to use the low_foreground color 0 < x < 1"
+         ),
+        ('low_foreground', 'FF0000', 'Font color on low battery'),
     ]
 
     def __init__(self, **config):
@@ -191,7 +187,7 @@ class Battery(_Battery):
     def _get_text(self):
         info = self._get_info()
         if info is False:
-            return 'Error'
+            return self.error_message
 
         # Set the charging character
         try:
@@ -208,6 +204,15 @@ class Battery(_Battery):
             elif info['stat'] == CHARGING:
                 char = self.charge_char
                 time = (info['full'] - info['now']) / info['power']
+            # if percent charge >0 and <50, but not discharging or charging, then return the current status
+            # TODO: make this configurable, don't just use 50% as arbitrary cut-off, maybe check if plugged in
+            elif info['now'] > 0 and \
+                    info['stat'] == UNKNOWN and \
+                    int(info['now'] / info['full']) != 1:
+                return '~' + str(int(info['now'] / info['full'] * 100)) + '%'
+            # battery is empty and not charging
+            elif info['now'] == 0 and info['stat'] == UNKNOWN:
+                return 'Empty'
             else:
                 return 'Full'
         except ZeroDivisionError:
@@ -241,9 +246,8 @@ class Battery(_Battery):
 
 
 class BatteryIcon(_Battery):
-    '''
-        Battery life indicator widget.
-    '''
+    """Battery life indicator widget."""
+
     orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
         ('theme_path', default_icon_path(), 'Path of the icons'),
@@ -326,7 +330,7 @@ class BatteryIcon(_Battery):
                 img = cairocffi.ImageSurface.create_from_png(path)
             except cairocffi.Error:
                 self.theme_path = None
-                self.qtile.log.warning('Battery Icon switching to text mode')
+                logger.warning('Battery Icon switching to text mode')
                 return
             input_width = img.get_width()
             input_height = img.get_height()
