@@ -38,6 +38,7 @@ import pickle
 import string
 from collections import deque
 
+from libqtile.log_utils import logger
 from . import base
 from .. import bar, command, hook, pangocffi, utils, xcbq, xkeysyms
 
@@ -71,9 +72,7 @@ class FileCompleter(object):
         self.lookup = None
 
     def complete(self, txt):
-        """
-        Returns the next completion for txt, or None if there is no completion.
-        """
+        """Returns the next completion for txt, or None if there is no completion"""
         if not self.lookup:
             self.lookup = []
             if txt == "" or txt[0] not in "~/":
@@ -164,9 +163,7 @@ class GroupCompleter(object):
         self.offset = None
 
     def actual(self):
-        """
-            Returns the current actual value.
-        """
+        """Returns the current actual value"""
         return self.thisfinal
 
     def reset(self):
@@ -174,9 +171,7 @@ class GroupCompleter(object):
         self.offset = -1
 
     def complete(self, txt):
-        """
-        Returns the next completion for txt, or None if there is no completion.
-        """
+        """Returns the next completion for txt, or None if there is no completion"""
         txt = txt.lower()
         if not self.lookup:
             self.lookup = []
@@ -204,9 +199,7 @@ class WindowCompleter(object):
         self.offset = None
 
     def actual(self):
-        """
-            Returns the current actual value.
-        """
+        """Returns the current actual value"""
         return self.thisfinal
 
     def reset(self):
@@ -214,9 +207,7 @@ class WindowCompleter(object):
         self.offset = -1
 
     def complete(self, txt):
-        """
-        Returns the next completion for txt, or None if there is no completion.
-        """
+        """Returns the next completion for txt, or None if there is no completion"""
         if not self.lookup:
             self.lookup = []
             for wid, window in self.qtile.windowMap.items():
@@ -236,22 +227,23 @@ class WindowCompleter(object):
 
 
 class CommandCompleter(object):
+    """
+    Parameters
+    ==========
+    _testing :
+        disables reloading of the lookup table to make testing possible.
+    """
+
     DEFAULTPATH = "/bin:/usr/bin:/usr/local/bin"
 
     def __init__(self, qtile, _testing=False):
-        """
-        _testing: disables reloading of the lookup table
-                  to make testing possible.
-        """
         self.lookup = None
-        self.offset = None
+        self.offset = -1
         self.thisfinal = None
         self._testing = _testing
 
     def actual(self):
-        """
-            Returns the current actual value.
-        """
+        """Returns the current actual value"""
         return self.thisfinal
 
     def executable(self, fpath):
@@ -262,9 +254,7 @@ class CommandCompleter(object):
         self.offset = -1
 
     def complete(self, txt):
-        """
-        Returns the next completion for txt, or None if there is no completion.
-        """
+        """Returns the next completion for txt, or None if there is no completion"""
         if not self.lookup:
             if not self._testing:
                 # Lookup is a set of (display value, actual value) tuples.
@@ -286,9 +276,10 @@ class CommandCompleter(object):
                             self.lookup.append((display, f))
                 else:
                     dirs = os.environ.get("PATH", self.DEFAULTPATH).split(":")
-                    for didx, d in enumerate(dirs):
+                    for d in dirs:
                         try:
-                            for cmd in glob.glob(os.path.join(d, "%s*" % txt)):
+                            d = os.path.expanduser(d)
+                            for cmd in glob.iglob(os.path.join(d, "%s*" % txt)):
                                 if self.executable(cmd):
                                     self.lookup.append(
                                         (
@@ -310,13 +301,13 @@ class CommandCompleter(object):
 
 
 class Prompt(base._TextBox):
-    """
-        A widget that prompts for user input. Input should be started using the
-        .startInput method on this class.
+    """A widget that prompts for user input
+
+    Input should be started using the ``.startInput()`` method on this class.
     """
     completers = {
         "file": FileCompleter,
-        "qsh": QshCompleter,
+        "qshell": QshCompleter,
         "cmd": CommandCompleter,
         "group": GroupCompleter,
         "window": WindowCompleter,
@@ -333,7 +324,7 @@ class Prompt(base._TextBox):
                  "Commands to keep in history. 0 for no limit."),
                 ("bell_style", "audible",
                  "Alert at the begin/end of the command history. " +
-                 "Posible values: 'audible', 'visual' and None."),
+                 "Possible values: 'audible', 'visual' and None."),
                 ("visual_bell_color", "ff0000",
                  "Color for the visual bell (changes prompt background)."),
                 ("visual_bell_time", 0.2,
@@ -371,10 +362,18 @@ class Prompt(base._TextBox):
             self.original_background = self.background
         # If history record is on, get saved history or create history record
         if self.record_history:
-            self.history_path = os.path.expanduser('~/.qtile_history')
+            self.history_path = os.path.join(utils.get_cache_dir(),
+                                             'prompt_history')
             if os.path.exists(self.history_path):
                 with open(self.history_path, 'rb') as f:
-                    self.history = pickle.load(f)
+                    try:
+                        self.history = pickle.load(f)
+                    except:
+                        # unfortunately, pickle doesn't wrap its errors, so we
+                        # can't detect what's a pickle error and what's not.
+                        logger.exception("failed to load prompt history")
+                        self.history = {x: deque(maxlen=self.max_history)
+                                        for x in self.completers if x}
                     if self.max_history != \
                        self.history[list(self.history)[0]].maxlen:
                         self.history = {x: deque(copy.copy(self.history[x]),
@@ -396,22 +395,29 @@ class Prompt(base._TextBox):
 
     def startInput(self, prompt, callback,
                    complete=None, strict_completer=False):
-        """
-            complete: Tab-completion. Can be None, "cmd", "file", "group",
-            "qsh" or "window".
+        """Run the prompt
 
-            Displays a prompt and starts to take one line of keyboard input
-            from the user. When done, calls the callback with the input string
-            as argument. If history record is enabled, also allows to browse
-            between previous commands with ↑ and ↓, and execute them
-            (untouched or modified). When historial is exhausted, fires an
-            alert. It tries to mimic, in some way, the shell behavior.
+        Displays a prompt and starts to take one line of keyboard input from
+        the user. When done, calls the callback with the input string as
+        argument. If history record is enabled, also allows to browse between
+        previous commands with ↑ and ↓, and execute them (untouched or
+        modified). When history is exhausted, fires an alert. It tries to
+        mimic, in some way, the shell behavior.
 
-            prompt = text displayed at the prompt, e.g. "spawn: "
-            callback = function to call with returned value.
-            complete = completer to use.
-            strict_completer = When True the return value wil be the exact
-                               completer result where available.
+        Parameters
+        ==========
+        complete :
+            Tab-completion. Can be None, "cmd", "file", "group", "qshell" or
+            "window".
+        prompt :
+            text displayed at the prompt, e.g. "spawn: "
+        callback :
+            function to call with returned value.
+        complete :
+            completer to use.
+        strict_completer :
+            When True the return value wil be the exact completer result where
+            available.
         """
 
         if self.cursor and self.cursorblink and not self.active:
@@ -450,7 +456,7 @@ class Prompt(base._TextBox):
 
     def _highlight_text(self, text):
         color = utils.hex(self.cursor_color)
-        text = '<span foreground="{}">{}</span>'.format(color, text)
+        text = '<span foreground="{0}">{1}</span>'.format(color, text)
         if self.show_cursor:
             text = '<u>{}</u>'.format(text)
         return text
@@ -466,17 +472,17 @@ class Prompt(base._TextBox):
                 for text in (txt1, txt2, txt3):
                     text = pangocffi.markup_escape_text(text)
                 txt2 = self._highlight_text(txt2)
-                self.text = "{}{}{}{}".format(txt1, txt2, txt3, cursor)
+                self.text = "{0}{1}{2}{3}".format(txt1, txt2, txt3, cursor)
             else:
                 self.text = pangocffi.markup_escape_text(self.text)
-                self.text = self.text + self._highlight_text(cursor)
+                self.text += self._highlight_text(cursor)
             self.text = self.display + self.text
         else:
             self.text = ""
         self.bar.draw()
 
     def _trigger_complete(self):
-        # Trigger the autocompletion in user input
+        # Trigger the auto completion in user input
         self.userInput = self.completer.complete(self.userInput)
         self.cursor_position = len(self.userInput)
 
@@ -489,7 +495,7 @@ class Prompt(base._TextBox):
             self.position = len(self.completer_history)
 
     def _insert_before_cursor(self, charcode):
-        # Insert a caracter (given their charcode) in input, before the cursor
+        # Insert a character (given their charcode) in input, before the cursor
         txt1 = self.userInput[:self.cursor_position]
         txt2 = self.userInput[self.cursor_position:]
         self.userInput = txt1 + chr(charcode) + txt2
@@ -633,9 +639,7 @@ class Prompt(base._TextBox):
         self.handle_KeyPress(d)
 
     def cmd_info(self):
-        """
-            Returns a dictionary of info for this object.
-        """
+        """Returns a dictionary of info for this object"""
         return dict(
             name=self.name,
             width=self.width,
