@@ -37,7 +37,7 @@ import cairocffi
 
 from . import base
 from .. import bar
-from six import u
+from libqtile.log_utils import logger
 
 __all__ = [
     'Volume',
@@ -47,10 +47,10 @@ re_vol = re.compile('\[(\d?\d?\d?)%\]')
 
 
 class Volume(base._TextBox):
-    '''
-        Widget that display and change volume if theme_path is set it draw
-        widget as icons.
-    '''
+    """Widget that display and change volume
+
+    If theme_path is set it draw widget as icons.
+    """
     orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
         ("cardid", None, "Card Id"),
@@ -139,20 +139,20 @@ class Volume(base._TextBox):
                 img_name = 'audio-volume-low'
             elif self.volume < 80:
                 img_name = 'audio-volume-medium'
-            elif self.volume >= 80:
+            else:  # self.volume >= 80:
                 img_name = 'audio-volume-high'
 
             self.drawer.ctx.set_source(self.surfaces[img_name])
             self.drawer.ctx.paint()
         elif self.emoji:
             if self.volume <= 0:
-                self.text = u('\U0001f507')
+                self.text = u'\U0001f507'
             elif self.volume <= 30:
-                self.text = u('\U0001f508')
+                self.text = u'\U0001f508'
             elif self.volume < 80:
-                self.text = u('\U0001f509')
+                self.text = u'\U0001f509'
             elif self.volume >= 80:
-                self.text = u('\U0001f50a')
+                self.text = u'\U0001f50a'
         else:
             if self.volume == -1:
                 self.text = 'M'
@@ -174,7 +174,7 @@ class Volume(base._TextBox):
             except cairocffi.Error:
                 self.theme_path = None
                 self.length_type = bar.CALCULATED
-                self.qtile.log.exception('Volume switching to text mode')
+                logger.exception('Volume switching to text mode')
                 return
             input_width = img.get_width()
             input_height = img.get_height()
@@ -215,7 +215,7 @@ class Volume(base._TextBox):
         if volgroups:
             return int(volgroups.groups()[0])
         else:
-            # this shouldn't happend
+            # this shouldn't happen
             return -1
 
     def draw(self):
