@@ -24,12 +24,14 @@ import array
 import contextlib
 import inspect
 import traceback
+import warnings
 from xcffib.xproto import EventMask, StackMode, SetMode
 import xcffib.xproto
 
 from . import command
 from . import utils
 from . import hook
+from .log_utils import logger
 
 
 # ICCM Constants
@@ -108,27 +110,74 @@ _NET_WM_STATE_ADD = 1
 _NET_WM_STATE_TOGGLE = 2
 
 
+def _geometry_getter(attr):
+    def get_attr(self):
+        if getattr(self, "_" + attr) is None:
+            g = self.window.get_geometry()
+            # trigger the geometry setter on all these
+            self.x = g.x
+            self.y = g.y
+            self.width = g.width
+            self.height = g.height
+        return getattr(self, "_" + attr)
+    return get_attr
+
+
+def _geometry_setter(attr):
+    def f(self, value):
+        if not isinstance(value, int):
+            frame = inspect.currentframe()
+            stack_trace = traceback.format_stack(frame)
+            logger.error("!!!! setting %s to a non-int %s; please report this!", attr, value)
+            logger.error(''.join(stack_trace[:-1]))
+            value = int(value)
+        setattr(self, "_" + attr, value)
+    return f
+
+
+def _float_getter(attr):
+    def getter(self):
+        if self._float_info[attr] is not None:
+            return self._float_info[attr]
+
+        # we don't care so much about width or height, if not set, default to the window width/height
+        if attr in ('width', 'height'):
+            return getattr(self, attr)
+
+        raise AttributeError("Floating not yet configured yet")
+    return getter
+
+
+def _float_setter(attr):
+    def setter(self, value):
+        self._float_info[attr] = value
+    return setter
+
+
 class _Window(command.CommandObject):
+    _windowMask = None  # override in child class
+
     def __init__(self, window, qtile):
         self.window, self.qtile = window, qtile
         self.hidden = True
         self.group = None
         self.icons = {}
         window.set_attribute(eventmask=self._windowMask)
+
+        self._float_info = {
+            'x': None,
+            'y': None,
+            'width': None,
+            'height': None,
+        }
         try:
             g = self.window.get_geometry()
             self._x = g.x
             self._y = g.y
             self._width = g.width
             self._height = g.height
-            # note that _float_info x and y are
-            # really offsets, relative to screen x,y
-            self._float_info = {
-                'x': g.x,
-                'y': g.y,
-                'w': g.width,
-                'h': g.height,
-            }
+            self._float_info['width'] = g.width
+            self._float_info['height'] = g.height
         except xcffib.xproto.DrawableError:
             # Whoops, we were too early, so let's ignore it for now and get the
             # values on demand.
@@ -136,7 +185,7 @@ class _Window(command.CommandObject):
             self._y = None
             self._width = None
             self._height = None
-            self._float_info = None
+
         self.borderwidth = 0
         self.bordercolor = None
         self.name = "<no name>"
@@ -163,35 +212,6 @@ class _Window(command.CommandObject):
         }
         self.updateHints()
 
-    def _geometry_getter(attr):
-        def get_attr(self):
-            if getattr(self, "_" + attr) is None:
-                g = self.window.get_geometry()
-                self.x = g.x
-                self.y = g.y
-                self.width = g.width
-                self.height = g.height
-                # note that _float_info x and y are
-                # really offsets, relative to screen x,y
-                self._float_info = {
-                    'x': g.x, 'y': g.y,
-                    'w': g.width, 'h': g.height
-                }
-
-            return getattr(self, "_" + attr)
-        return get_attr
-
-    def _geometry_setter(attr):
-        def f(self, value):
-            if not isinstance(value, int) and attr != "_float_info":
-                frame = inspect.currentframe()
-                stack_trace = traceback.format_stack(frame)
-                self.qtile.log.error("!!!! setting %s to a non-int %s; please report this!", attr, value)
-                self.qtile.log.error(''.join(stack_trace[:-1]))
-                value = int(value)
-            setattr(self, "_" + attr, value)
-        return f
-
     x = property(fset=_geometry_setter("x"), fget=_geometry_getter("x"))
     y = property(fset=_geometry_setter("y"), fget=_geometry_getter("y"))
     width = property(
@@ -202,9 +222,22 @@ class _Window(command.CommandObject):
         fset=_geometry_setter("height"),
         fget=_geometry_getter("height")
     )
-    _float_info = property(
-        fset=_geometry_setter("_float_info"),
-        fget=_geometry_getter("_float_info")
+
+    float_x = property(
+        fset=_float_setter("x"),
+        fget=_float_getter("x")
+    )
+    float_y = property(
+        fset=_float_setter("y"),
+        fget=_float_getter("y")
+    )
+    float_width = property(
+        fset=_float_setter("width"),
+        fget=_float_getter("width")
+    )
+    float_height = property(
+        fset=_float_setter("height"),
+        fget=_float_getter("height")
     )
 
     def updateName(self):
@@ -215,9 +248,9 @@ class _Window(command.CommandObject):
         hook.fire("window_name_change")
 
     def updateHints(self):
-        """
-            update the local copy of the window's WM_HINTS
-            http://tronche.com/gui/x/icccm/sec-4.html#WM_HINTS
+        """Update the local copy of the window's WM_HINTS
+
+        See http://tronche.com/gui/x/icccm/sec-4.html#WM_HINTS
         """
         try:
             h = self.window.get_wm_hints()
@@ -285,10 +318,9 @@ class _Window(command.CommandObject):
 
         state = self.window.get_net_wm_state()
 
-        if state:
-            self.qtile.log.debug('_NET_WM_STATE: %s' % ','.join(state))
-            for s in triggered:
-                setattr(self, s, (s in state))
+        logger.debug('_NET_WM_STATE: %s', state)
+        for s in triggered:
+            setattr(self, s, (s in state))
 
     @property
     def urgent(self):
@@ -403,10 +435,9 @@ class _Window(command.CommandObject):
 
     def place(self, x, y, width, height, borderwidth, bordercolor,
               above=False, force=False, margin=None):
-        """
-            Places the window at the specified location with the given size.
+        """Places the window at the specified location with the given size.
 
-            if force is false, than it tries to obey hints
+        If force is false, than it tries to obey hints
         """
 
         # TODO: self.x/y/height/width are updated BEFORE
@@ -430,6 +461,11 @@ class _Window(command.CommandObject):
             width -= margin * 2
             height -= margin * 2
 
+        # save x and y float offset
+        if self.group is not None and self.group.screen is not None:
+            self.float_x = x - self.group.screen.x
+            self.float_y = y - self.group.screen.y
+
         self.x = x
         self.y = y
         self.width = width
@@ -437,11 +473,6 @@ class _Window(command.CommandObject):
         self.borderwidth = borderwidth
         self.bordercolor = bordercolor
 
-        # save x and y float offset
-        if self.group is not None and self.group.screen is not None:
-            self._float_info['x'] = x - self.group.screen.x
-            self._float_info['y'] = y - self.group.screen.y
-
         kwarg = dict(
             x=x,
             y=y,
@@ -461,9 +492,7 @@ class _Window(command.CommandObject):
             self.window.set_attribute(borderpixel=bordercolor)
 
     def send_configure_notify(self, x, y, width, height):
-        """
-        Send a synthetic ConfigureNotify
-        """
+        """Send a synthetic ConfigureNotify"""
 
         window = self.window.wid
         above_sibling = False
@@ -534,7 +563,18 @@ class _Window(command.CommandObject):
                     self.window.warp_pointer(self.width // 2, self.height // 2)
             except AttributeError:
                 pass
-        self.urgent = False
+
+        if self.urgent:
+            self.urgent = False
+
+            atom = self.qtile.conn.atoms["_NET_WM_STATE_DEMANDS_ATTENTION"]
+            state = list(self.window.get_property('_NET_WM_STATE', 'ATOM',
+                unpack=int))
+
+            if atom in state:
+                state.remove(atom)
+                self.window.set_property('_NET_WM_STATE', state)
+
         self.qtile.root.set_property("_NET_ACTIVE_WINDOW", self.window.wid)
         hook.fire("client_focus", self)
 
@@ -545,15 +585,11 @@ class _Window(command.CommandObject):
         return None
 
     def cmd_info(self):
-        """
-            Returns a dictionary of info for this object.
-        """
+        """Returns a dictionary of info for this object"""
         return self.info()
 
     def cmd_inspect(self):
-        """
-            Tells you more than you ever wanted to know about a window.
-        """
+        """Tells you more than you ever wanted to know about a window"""
         a = self.window.get_attributes()
         attrs = {
             "backing_store": a.backing_store,
@@ -600,9 +636,7 @@ class _Window(command.CommandObject):
 
 
 class Internal(_Window):
-    """
-        An internal window, that should not be managed by qtile.
-    """
+    """An internal window, that should not be managed by qtile"""
     _windowMask = EventMask.StructureNotify | \
         EventMask.PropertyChange | \
         EventMask.EnterWindow | \
@@ -613,7 +647,7 @@ class Internal(_Window):
         EventMask.KeyPress
 
     @classmethod
-    def create(klass, qtile, x, y, width, height, opacity=1.0):
+    def create(cls, qtile, x, y, width, height, opacity=1.0):
         win = qtile.conn.create_window(x, y, width, height)
         win.set_property("QTILE_INTERNAL", 1)
         i = Internal(win, qtile)
@@ -622,7 +656,7 @@ class Internal(_Window):
         return i
 
     def __repr__(self):
-        return "Internal(%s, %s)" % (self.name, self.window.wid)
+        return "Internal(%r, %s)" % (self.name, self.window.wid)
 
     def kill(self):
         self.qtile.conn.conn.core.DestroyWindow(self.window.wid)
@@ -632,9 +666,7 @@ class Internal(_Window):
 
 
 class Static(_Window):
-    """
-        An internal window, that should not be managed by qtile.
-    """
+    """An internal window, that should not be managed by qtile"""
     _windowMask = EventMask.StructureNotify | \
         EventMask.PropertyChange | \
         EventMask.EnterWindow | \
@@ -698,7 +730,7 @@ class Static(_Window):
             self.update_strut()
 
     def __repr__(self):
-        return "Static(%s)" % self.name
+        return "Static(%r)" % self.name
 
 
 class Window(_Window):
@@ -714,9 +746,16 @@ class Window(_Window):
         self._group = None
         self.updateName()
         # add to group by position according to _NET_WM_DESKTOP property
+        group = None
         index = window.get_wm_desktop()
         if index is not None and index < len(qtile.groups):
             group = qtile.groups[index]
+        elif index is None:
+            transient_for = window.get_wm_transient_for()
+            win = qtile.windowMap.get(transient_for)
+            if win is not None:
+                group = win._group
+        if group is not None:
             group.add(self)
             self._group = group
             if group != qtile.currentScreen.group:
@@ -739,7 +778,7 @@ class Window(_Window):
                     self.qtile.groups.index(group)
                 )
             except xcffib.xproto.WindowError:
-                self.qtile.log.exception("whoops, got error setting _NET_WM_DESKTOP, too early?")
+                logger.exception("whoops, got error setting _NET_WM_DESKTOP, too early?")
         self._group = group
 
     @property
@@ -753,7 +792,37 @@ class Window(_Window):
     @floating.setter
     def floating(self, do_float):
         if do_float and self._float_state == NOT_FLOATING:
-            self.enablefloating()
+            if self.group and self.group.screen:
+                screen = self.group.screen
+                self._enablefloating(
+                    screen.x + self.float_x, screen.y + self.float_y, self.float_width, self.float_height
+                )
+            else:
+                # if we are setting floating early, e.g. from a hook, we don't have a screen yet
+                self._float_state = FLOATING
+        elif (not do_float) and self._float_state != NOT_FLOATING:
+            if self._float_state == FLOATING:
+                # store last size
+                self.float_width = self.width
+                self.float_height = self.height
+            self._float_state = NOT_FLOATING
+            self.group.mark_floating(self, False)
+            hook.fire('float_change')
+
+    def toggle_floating(self):
+        self.floating = not self.floating
+
+    def togglefloating(self):
+        warnings.warn("togglefloating is deprecated, use toggle_floating", DeprecationWarning)
+        self.toggle_floating()
+
+    def enablefloating(self):
+        warnings.warn("enablefloating is deprecated, use floating=True", DeprecationWarning)
+        self.floating = True
+
+    def disablefloating(self):
+        warnings.warn("disablefloating is deprecated, use floating=False", DeprecationWarning)
+        self.floating = False
 
     @property
     def fullscreen(self):
@@ -761,12 +830,37 @@ class Window(_Window):
 
     @fullscreen.setter
     def fullscreen(self, do_full):
+        atom = set([self.qtile.conn.atoms["_NET_WM_STATE_FULLSCREEN"]])
+        prev_state = set(self.window.get_property('_NET_WM_STATE', 'ATOM', unpack=int))
+
         if do_full:
-            if self._float_state != FULLSCREEN:
-                self.enablemaximize(state=FULLSCREEN)
+            screen = self.group.screen or \
+                self.qtile.find_closest_screen(self.x, self.y)
+
+            self._enablefloating(
+                screen.x,
+                screen.y,
+                screen.width,
+                screen.height,
+                new_float_state=FULLSCREEN
+            )
+            state = prev_state | atom
         else:
             if self._float_state == FULLSCREEN:
-                self.disablefloating()
+                self.floating = False
+                state = prev_state - atom
+            else:
+                state = prev_state
+
+        if prev_state != state:
+            self.window.set_property('_NET_WM_STATE', list(state))
+
+    def toggle_fullscreen(self):
+        self.fullscreen = not self.fullscreen
+
+    def togglefullscreen(self):
+        warnings.warn("togglefullscreen is deprecated, use toggle_fullscreen", DeprecationWarning)
+        self.toggle_fullscreen()
 
     @property
     def maximized(self):
@@ -775,11 +869,30 @@ class Window(_Window):
     @maximized.setter
     def maximized(self, do_maximize):
         if do_maximize:
-            if self._float_state != MAXIMIZED:
-                self.enablemaximize()
+            screen = self.group.screen or \
+                self.qtile.find_closest_screen(self.x, self.y)
+
+            self._enablefloating(
+                screen.dx,
+                screen.dy,
+                screen.dwidth,
+                screen.dheight,
+                new_float_state=MAXIMIZED
+            )
         else:
             if self._float_state == MAXIMIZED:
-                self.disablefloating()
+                self.floating = False
+
+    def enablemaximize(self, state=MAXIMIZED):
+        warnings.warn("enablemaximize is deprecated, use maximized=True", DeprecationWarning)
+        self.maximized = True
+
+    def toggle_maximize(self, state=MAXIMIZED):
+        self.maximized = not self.maximized
+
+    def togglemaximize(self):
+        warnings.warn("togglemaximize is deprecated, use toggle_maximize", DeprecationWarning)
+        self.toggle_maximize()
 
     @property
     def minimized(self):
@@ -789,18 +902,29 @@ class Window(_Window):
     def minimized(self, do_minimize):
         if do_minimize:
             if self._float_state != MINIMIZED:
-                self.enableminimize()
+                self._enablefloating(new_float_state=MINIMIZED)
         else:
             if self._float_state == MINIMIZED:
-                self.disablefloating()
+                self.floating = False
+
+    def enableminimize(self):
+        warnings.warn("enableminimized is deprecated, use minimized=True", DeprecationWarning)
+        self.minimized = True
+
+    def toggle_minimize(self):
+        self.minimized = not self.minimized
+
+    def toggleminimize(self):
+        warnings.warn("toggleminimize is deprecated, use toggle_minimize", DeprecationWarning)
+        self.toggle_minimize()
 
     def static(self, screen, x=None, y=None, width=None, height=None):
-        """
-            Makes this window a static window, attached to a Screen. If any of
-            the arguments are left unspecified, the values given by the window
-            itself are used instead. So, for a window that's aware of its
-            appropriate size and location (like dzen), you don't have to
-            specify anything.
+        """Makes this window a static window, attached to a Screen
+
+        If any of the arguments are left unspecified, the values given by the
+        window itself are used instead. So, for a window that's aware of its
+        appropriate size and location (like dzen), you don't have to specify
+        anything.
         """
         self.defunct = True
         screen = self.qtile.screens[screen]
@@ -836,8 +960,8 @@ class Window(_Window):
 
         screen = self.qtile.find_closest_screen(self.x, self.y)
         if self.group and screen is not None and screen != self.group.screen:
-            self.group.remove(self)
-            screen.group.add(self)
+            self.group.remove(self, force=True)
+            screen.group.add(self, force=True)
             self.qtile.toScreen(screen.index)
 
         self._reconfigure_floating()
@@ -848,89 +972,22 @@ class Window(_Window):
     def getposition(self):
         return (self.x, self.y)
 
-    def toggleminimize(self):
-        if self.minimized:
-            self.disablefloating()
-        else:
-            self.enableminimize()
-
-    def enableminimize(self):
-        self._enablefloating(new_float_state=MINIMIZED)
-
-    def togglemaximize(self, state=MAXIMIZED):
-        if self._float_state == state:
-            self.disablefloating()
-        else:
-            self.enablemaximize(state)
-
-    def enablemaximize(self, state=MAXIMIZED):
-        screen = self.group.screen or self.qtile.find_closest_screen(
-            self.x,
-            self.y
-        )
-
-        if state == MAXIMIZED:
-            self._enablefloating(
-                screen.dx,
-                screen.dy,
-                screen.dwidth,
-                screen.dheight,
-                new_float_state=state
-            )
-        elif state == FULLSCREEN:
-            self._enablefloating(
-                screen.x,
-                screen.y,
-                screen.width,
-                screen.height,
-                new_float_state=state
-            )
-
-    def togglefloating(self):
-        if self.floating:
-            self.disablefloating()
-        else:
-            self.enablefloating()
-
     def _reconfigure_floating(self, new_float_state=FLOATING):
         if new_float_state == MINIMIZED:
             self.state = IconicState
             self.hide()
         else:
-            # make sure x, y is on the screen
-            screen = self.qtile.find_closest_screen(self.x, self.y)
-            if screen is not None and \
-                    self.group is not None and \
-                    self.group.screen is not None and \
-                    screen != self.group.screen:
-                x = self.group.screen.x
-                y = self.group.screen.y
-            else:
-                x = self.x
-                y = self.y
+            width = max(self.width, self.hints.get('min_width', 0))
+            height = max(self.height, self.hints.get('min_height', 0))
 
-            if self.width < self.hints.get('min_width', 0):
-                width = self.hints['min_width']
-            else:
-                width = self.width
+            if self.hints['base_width'] and self.hints['width_inc']:
+                width -= (width - self.hints['base_width']) % self.hints['width_inc']
 
-            if self.height < self.hints.get('min_height', 0):
-                height = self.hints['min_height']
-            else:
-                height = self.height
-
-            if self.hints.get('width_inc', 0):
-                width = (width -
-                         ((width - self.hints['base_width']) %
-                          self.hints['width_inc']))
-
-            if self.hints.get('height_inc', 0):
-                height = (height -
-                          ((height - self.hints['base_height'])
-                           % self.hints['height_inc']))
+            if self.hints['base_height'] and self.hints['height_inc']:
+                height -= (height - self.hints['base_height']) % self.hints['height_inc']
 
             self.place(
-                x, y,
+                self.x, self.y,
                 width, height,
                 self.borderwidth,
                 self.bordercolor,
@@ -951,28 +1008,14 @@ class Window(_Window):
             self.height = h
         self._reconfigure_floating(new_float_state=new_float_state)
 
-    def enablefloating(self):
-        fi = self._float_info
-        self._enablefloating(fi['x'], fi['y'], fi['w'], fi['h'])
-
-    def disablefloating(self):
-        if self._float_state != NOT_FLOATING:
-            if self._float_state == FLOATING:
-                # store last size
-                fi = self._float_info
-                fi['w'] = self.width
-                fi['h'] = self.height
-            self._float_state = NOT_FLOATING
-            self.group.mark_floating(self, False)
-            hook.fire('float_change')
-
-    def togroup(self, groupName):
-        """
-            Move window to a specified group.
-        """
-        group = self.qtile.groupMap.get(groupName)
-        if group is None:
-            raise command.CommandError("No such group: %s" % groupName)
+    def togroup(self, groupName=None):
+        """Move window to a specified group"""
+        if groupName is None:
+            group = self.qtile.currentGroup
+        else:
+            group = self.qtile.groupMap.get(groupName)
+            if group is None:
+                raise command.CommandError("No such group: %s" % groupName)
 
         if self.group is not group:
             self.hide()
@@ -987,16 +1030,17 @@ class Window(_Window):
             group.add(self)
 
     def match(self, wname=None, wmclass=None, role=None):
-        """
-            Match window against given attributes.
+        """Match window against given attributes.
 
-            - wname matches against the window name or title, that is,
-            either ``_NET_WM_VISIBLE_NAME``, ``_NET_WM_NAME``, ``WM_NAME``.
-
-            - wmclass matches against any of the two values in the
-            ``WM_CLASS`` property
-
-            - role matches against the ``WM_WINDOW_ROLE`` property
+        Parameters
+        ==========
+        wname :
+            matches against the window name or title, that is, either
+            ``_NET_WM_VISIBLE_NAME``, ``_NET_WM_NAME``, ``WM_NAME``.
+        wmclass :
+            matches against any of the two values in the ``WM_CLASS`` property
+        role :
+            matches against the ``WM_WINDOW_ROLE`` property
         """
         if not (wname or wmclass or role):
             raise TypeError(
@@ -1053,9 +1097,7 @@ class Window(_Window):
         return False
 
     def update_wm_net_icon(self):
-        """
-            Set a dict with the icons of the window
-        """
+        """Set a dict with the icons of the window"""
 
         icon = self.window.get_property('_NET_WM_ICON', 'CARDINAL')
         if not icon:
@@ -1094,19 +1136,12 @@ class Window(_Window):
 
         opcode = event.type
         data = event.data
-        if atoms["_NET_WM_STATE"] == opcode and \
-                self.qtile.config.auto_fullscreen:
-            fullscreen_atom = atoms["_NET_WM_STATE_FULLSCREEN"]
-
+        if atoms["_NET_WM_STATE"] == opcode:
             prev_state = self.window.get_property(
                 '_NET_WM_STATE',
                 'ATOM',
                 unpack=int
             )
-            if not prev_state:
-                prev_state = []
-                if self.fullscreen:
-                    prev_state.append(fullscreen_atom)
 
             current_state = set(prev_state)
 
@@ -1123,14 +1158,28 @@ class Window(_Window):
                 elif action == _NET_WM_STATE_TOGGLE:
                     current_state ^= set([prop])  # toggle :D
 
-            # add support for additional flags here
-            self.fullscreen = (fullscreen_atom in current_state)
-
             self.window.set_property('_NET_WM_STATE', list(current_state))
+        elif atoms["_NET_ACTIVE_WINDOW"] == opcode:
+            source = data.data32[0]
+            if source == 2:  # XCB_EWMH_CLIENT_SOURCE_TYPE_NORMAL
+                logger.info("Focusing window by pager")
+                self.qtile.currentScreen.setGroup(self.group)
+                self.group.focus(self)
+            else:  # XCB_EWMH_CLIENT_SOURCE_TYPE_OTHER
+                focus_behavior = self.qtile.config.focus_on_window_activation
+                if focus_behavior == "focus" or (focus_behavior == "smart" and self.group.screen and self.group.screen == self.qtile.currentScreen):
+                    logger.info("Focusing window")
+                    self.qtile.currentScreen.setGroup(self.group)
+                    self.group.focus(self)
+                elif focus_behavior == "urgent" or (focus_behavior == "smart" and not self.group.screen):
+                    logger.info("Setting urgent flag for window")
+                    self.urgent = True
+                else:
+                    logger.info("Ignoring focus request")
 
     def handle_PropertyNotify(self, e):
         name = self.qtile.conn.atoms.get_name(e.atom)
-        self.qtile.log.debug("PropertyNotifyEvent: %s" % name)
+        logger.debug("PropertyNotifyEvent: %s", name)
         if name == "WM_TRANSIENT_FOR":
             pass
         elif name == "WM_HINTS":
@@ -1170,7 +1219,7 @@ class Window(_Window):
                     self.group.currentWindow != self:
                 self.group.focus(self, False)
         else:
-            self.qtile.log.info("Unknown window property: %s" % name)
+            logger.info("Unknown window property: %s", name)
         return False
 
     def _items(self, name):
@@ -1193,50 +1242,51 @@ class Window(_Window):
             return self.group.screen
 
     def __repr__(self):
-        return "Window(%s)" % self.name
+        return "Window(%r)" % self.name
 
     def cmd_static(self, screen, x, y, width, height):
         self.static(screen, x, y, width, height)
 
     def cmd_kill(self):
-        """
-            Kill this window. Try to do this politely if the client support
-            this, otherwise be brutal.
+        """Kill this window
+
+        Try to do this politely if the client support
+        this, otherwise be brutal.
         """
         self.kill()
 
-    def cmd_togroup(self, groupName):
-        """
-            Move window to a specified group.
+    def cmd_togroup(self, groupName=None):
+        """Move window to a specified group.
 
-            Examples:
+        If groupName is not specified, we assume the current group
 
-                togroup("a")
+        Examples
+        ========
+
+        Move window to current group::
+
+            togroup()
+
+        Move window to group "a"::
+
+            togroup("a")
         """
         self.togroup(groupName)
 
     def cmd_move_floating(self, dx, dy, curx, cury):
-        """
-            Move window by dx and dy
-        """
+        """Move window by dx and dy"""
         self.tweak_float(dx=dx, dy=dy)
 
     def cmd_resize_floating(self, dw, dh, curx, cury):
-        """
-            Add dw and dh to size of window
-        """
+        """Add dw and dh to size of window"""
         self.tweak_float(dw=dw, dh=dh)
 
     def cmd_set_position_floating(self, x, y, curx, cury):
-        """
-            Move window to x and y
-        """
+        """Move window to x and y"""
         self.tweak_float(x=x, y=y)
 
     def cmd_set_size_floating(self, w, h, curx, cury):
-        """
-            Set window dimensions to w and h
-        """
+        """Set window dimensions to w and h"""
         self.tweak_float(w=w, h=h)
 
     def cmd_get_position(self):
@@ -1246,40 +1296,40 @@ class Window(_Window):
         return self.getsize()
 
     def cmd_toggle_floating(self):
-        self.togglefloating()
-
-    def cmd_disable_floating(self):
-        self.disablefloating()
+        self.toggle_floating()
 
     def cmd_enable_floating(self):
-        self.enablefloating()
+        self.floating = True
+
+    def cmd_disable_floating(self):
+        self.floating = False
 
     def cmd_toggle_maximize(self):
-        self.togglemaximize()
-
-    def cmd_disable_maximimize(self):
-        self.disablefloating()
+        self.toggle_maximize()
 
     def cmd_enable_maximize(self):
-        self.enablemaximize()
+        self.maximize = True
+
+    def cmd_disable_maximize(self):
+        self.maximize = False
 
     def cmd_toggle_fullscreen(self):
-        self.togglemaximize(state=FULLSCREEN)
+        self.toggle_fullscreen()
 
     def cmd_enable_fullscreen(self):
-        self.enablemaximize(state=FULLSCREEN)
+        self.fullscreen = True
 
     def cmd_disable_fullscreen(self):
-        self.disablefloating()
+        self.fullscreen = False
 
     def cmd_toggle_minimize(self):
-        self.toggleminimize()
+        self.toggle_minimize()
 
     def cmd_enable_minimize(self):
-        self.enableminimize()
+        self.minimize = True
 
     def cmd_disable_minimize(self):
-        self.disablefloating()
+        self.minimize = False
 
     def cmd_bring_to_front(self):
         if self.floating:
