@@ -47,10 +47,20 @@ import xcffib.xinerama
 import xcffib.xproto
 
 from . import xkeysyms
+from .log_utils import logger
 from .xcursors import Cursors
 
 keysyms = xkeysyms.keysyms
 
+
+def rdict(d):
+    r = {}
+    for k, v in d.items():
+        r.setdefault(v, []).append(k)
+    return r
+
+rkeysyms = rdict(xkeysyms.keysyms)
+
 # These should be in xpyb:
 ModMasks = {
     "shift": 1 << 0,
@@ -159,7 +169,8 @@ PropertyMap = {
     "_NET_WM_STATE_MODAL": ("ATOM", 32),
     "_NET_WM_STATE_HIDDEN": ("ATOM", 32),
     "_NET_WM_STATE_DEMANDS_ATTENTION": ("ATOM", 32),
-
+    # Xembed
+    "_XEMBED_INFO": ("_XEMBED_INFO", 32),
     # ICCCM
     "WM_STATE": ("WM_STATE", 32),
     # Qtile-specific properties
@@ -207,7 +218,7 @@ class MaskMap(object):
         A general utility class that encapsulates the way the mask/value idiom
         works in xpyb. It understands a special attribute _maskvalue on
         objects, which will be used instead of the object value if present.
-        This lets us passin a Font object, rather than Font.fid, for example.
+        This lets us pass in a Font object, rather than Font.fid, for example.
     """
     def __init__(self, obj):
         self.mmap = []
@@ -297,7 +308,7 @@ class Screen(_Wrapper):
 class PseudoScreen(object):
     """
         This may be a Xinerama screen or a RandR CRTC, both of which are
-        rectagular sections of an actual Screen.
+        rectangular sections of an actual Screen.
     """
     def __init__(self, conn, x, y, width, height):
         self.conn = conn
@@ -395,9 +406,7 @@ class Window(object):
         self.wid = wid
 
     def _propertyString(self, r):
-        """
-            Extract a string from a window property reply message.
-        """
+        """Extract a string from a window property reply message"""
         return r.value.to_string()
 
     def _propertyUTF8(self, r):
@@ -417,9 +426,7 @@ class Window(object):
         )
 
     def warp_pointer(self, x, y):
-        """
-            Warps the pointer to the location `x`, `y` on the window
-        """
+        """Warps the pointer to the location `x`, `y` on the window"""
         self.conn.conn.core.WarpPointer(
             0, self.wid,  # src_window, dst_window
             0, 0,         # src_x, src_y
@@ -428,12 +435,12 @@ class Window(object):
         )
 
     def get_name(self):
-        """
-            Tries to retrieve a canonical window name. We test the following
-            properties in order of preference:
-                - _NET_WM_VISIBLE_NAME
-                - _NET_WM_NAME
-                - WM_NAME.
+        """Tries to retrieve a canonical window name.
+
+        We test the following properties in order of preference:
+            - _NET_WM_VISIBLE_NAME
+            - _NET_WM_NAME
+            - WM_NAME.
         """
         r = self.get_property("_NET_WM_VISIBLE_NAME", "UTF8_STRING")
         if r:
@@ -457,14 +464,14 @@ class Window(object):
             flags = set(k for k, v in HintsFlags.items() if l[0] & v)
             return dict(
                 flags=flags,
-                input=l[1],
-                initial_state=l[2],
-                icon_pixmap=l[3],
-                icon_window=l[4],
-                icon_x=l[5],
-                icon_y=l[6],
-                icon_mask=l[7],
-                window_group=l[8]
+                input=l[1] if "InputHint" in flags else None,
+                initial_state=l[2] if "StateHing" in flags else None,
+                icon_pixmap=l[3] if "IconPixmapHint" in flags else None,
+                icon_window=l[4] if "IconWindowHint" in flags else None,
+                icon_x=l[5] if "IconPositionHint" in flags else None,
+                icon_y=l[6] if "IconPositionHint" in flags else None,
+                icon_mask=l[7] if "IconMaskHint" in flags else None,
+                window_group=l[8] if 'WindowGroupHint' in flags else None,
             )
 
     def get_wm_normal_hints(self):
@@ -492,19 +499,20 @@ class Window(object):
 
     def get_wm_protocols(self):
         l = self.get_property("WM_PROTOCOLS", "ATOM", unpack=int)
-        return set(self.conn.atoms.get_name(i) for i in l)
+        if l is not None:
+            return set(self.conn.atoms.get_name(i) for i in l)
+        return set()
 
     def get_wm_state(self):
         return self.get_property("WM_STATE", xcffib.xproto.GetPropertyType.Any, unpack=int)
 
     def get_wm_class(self):
-        """
-            Return an (instance, class) tuple if WM_CLASS exists, or None.
-        """
+        """Return an (instance, class) tuple if WM_CLASS exists, or None"""
         r = self.get_property("WM_CLASS", "STRING")
         if r:
             s = self._propertyString(r)
             return tuple(s.strip("\0").split("\0"))
+        return tuple()
 
     def get_wm_window_role(self):
         r = self.get_property("WM_WINDOW_ROLE", "STRING")
@@ -564,7 +572,7 @@ class Window(object):
 
     def configure(self, **kwargs):
         """
-            Arguments can be: x, y, width, height, border, sibling, stackmode
+        Arguments can be: x, y, width, height, border, sibling, stackmode
         """
         mask, values = ConfigureMasks(**kwargs)
         # hack for negative numbers
@@ -586,9 +594,11 @@ class Window(object):
 
     def set_property(self, name, value, type=None, format=None):
         """
-            name: String Atom name
-            type: String Atom name
-            format: 8, 16, 32
+        Parameters
+        ==========
+        name : String Atom name
+        type : String Atom name
+        format : 8, 16, 32
         """
         if name in PropertyMap:
             if type or format:
@@ -614,30 +624,35 @@ class Window(object):
                     value = value.encode('utf-8')
             else:
                 # if this runs without error, the value is already a list, don't wrap it
-                six.next(iter(value))
+                next(iter(value))
         except StopIteration:
             # The value was an iterable, just empty
             value = []
         except TypeError:
-            # the value wasn't an interable and wasn't a string, so let's
+            # the value wasn't an iterable and wasn't a string, so let's
             # wrap it.
             value = [value]
 
-        self.conn.conn.core.ChangePropertyChecked(
-            xcffib.xproto.PropMode.Replace,
-            self.wid,
-            self.conn.atoms[name],
-            self.conn.atoms[type],
-            format,  # Format - 8, 16, 32
-            len(value),
-            value
-        ).check()
+        try:
+            self.conn.conn.core.ChangePropertyChecked(
+                xcffib.xproto.PropMode.Replace,
+                self.wid,
+                self.conn.atoms[name],
+                self.conn.atoms[type],
+                format,  # Format - 8, 16, 32
+                len(value),
+                value
+            ).check()
+        except xcffib.xproto.WindowError:
+            logger.warning(
+                'X error in SetProperty (wid=%r, prop=%r), ignoring',
+                self.wid, name)
 
     def get_property(self, prop, type=None, unpack=None):
-        """
-            Return the contents of a property as a GetPropertyReply. If unpack
-            is specified, a tuple of values is returned.  The type to unpack,
-            either `str` or `int` must be specified.
+        """Return the contents of a property as a GetPropertyReply
+
+        If unpack is specified, a tuple of values is returned.  The type to
+        unpack, either `str` or `int` must be specified.
         """
         if type is None:
             if prop not in PropertyMap:
@@ -647,16 +662,24 @@ class Window(object):
             else:
                 type, _ = PropertyMap[prop]
 
-        r = self.conn.conn.core.GetProperty(
-            False, self.wid,
-            self.conn.atoms[prop]
-            if isinstance(prop, six.string_types)
-            else prop,
-            self.conn.atoms[type]
-            if isinstance(type, six.string_types)
-            else type,
-            0, (2 ** 32) - 1
-        ).reply()
+        try:
+            r = self.conn.conn.core.GetProperty(
+                False, self.wid,
+                self.conn.atoms[prop]
+                if isinstance(prop, six.string_types)
+                else prop,
+                self.conn.atoms[type]
+                if isinstance(type, six.string_types)
+                else type,
+                0, (2 ** 32) - 1
+            ).reply()
+        except (xcffib.xproto.WindowError, xcffib.xproto.AccessError):
+            logger.warning(
+                'X error in GetProperty (wid=%r, prop=%r), ignoring',
+                self.wid, prop)
+            if unpack:
+                return []
+            return None
 
         if not r.value_len:
             if unpack:
@@ -691,9 +714,7 @@ class Window(object):
         return GC(self.conn, gid)
 
     def ungrab_key(self, key, modifiers):
-        """
-            Passing None means any key, or any modifier.
-        """
+        """Passing None means any key, or any modifier"""
         if key is None:
             key = xcffib.xproto.Atom.Any
         if modifiers is None:
@@ -712,9 +733,7 @@ class Window(object):
         )
 
     def ungrab_button(self, button, modifiers):
-        """
-            Passing None means any key, or any modifier.
-        """
+        """Passing None means any key, or any modifier"""
         if button is None:
             button = xcffib.xproto.Atom.Any
         if modifiers is None:
@@ -772,10 +791,8 @@ class Font(object):
         return self.fid
 
     def text_extents(self, s):
-        s = s + "aaa"
-        print(s)
+        s += "aaa"
         x = self.conn.conn.core.QueryTextExtents(self.fid, len(s), s).reply()
-        print(x)
         return x
 
 
@@ -829,6 +846,10 @@ class Connection(object):
         self.modmap = None
         self.refresh_modmap()
 
+    def finalize(self):
+        self.cursors.finalize()
+        self.disconnect()
+
     def refresh_keymap(self, first=None, count=None):
         if first is None:
             first = self.setup.min_keycode
@@ -856,9 +877,7 @@ class Connection(object):
         self.modmap = modmap
 
     def get_modifier(self, keycode):
-        """
-            Return the modifier matching keycode.
-        """
+        """Return the modifier matching keycode"""
         for n, l in self.modmap.items():
             if keycode in l:
                 return n
